import unittest
import yaml
from pika_client.factory import create_persistent_async_consumer, create_persistent_async_publisher
import time

with open("test/config/connectors.yaml", "r") as fdr:
    config = yaml.load(fdr)

connection_parameters = config["publishers"]["default"]["connection"]
consumer_route_setup_conf = config["consumers"]["default"]["route"]
publisher_route_setup_conf = config["publishers"]["default"]["route"]


class TestWorkQueue(unittest.TestCase):
    def setUp(self):
        self._publisher = create_persistent_async_publisher(connection_parameters, publisher_route_setup_conf)
        self._publisher.start()
        time.sleep(5)
        self._publisher._channel.queue_purge(callback=None, queue=config["publishers"]["default"]["route"]["routing_key"])
        # time.sleep(5)

    def tearDown(self):
        self._publisher.stop()

    def on_message(self, body):
        print(body)


    # def test_stop_publisher(self):
    #     self._stop_connector(self._publisher)
    #
    # def test_stop_consumer(self):
    #     self._stop_connector(self._consumer)
    #
    # @staticmethod
    # def _stop_connector(connector):
    #     connector.start()
    #     time.sleep(5)
    #     connector.stop()
    #     time.sleep(10)





    def test_message(self):
        # self._consumer.start()
        for x in range(1000):
            self._publisher.send(
                {"product_name": "",
                 "photo_url": "",
                 "barcode": "",
                 "sku": "",
                 "price_cents": "",
                 "producer": ""
                 },
                exchange=config["publishers"]["default"]["route"]["exchange"],
                routing_key=config["publishers"]["default"]["route"]["routing_key"]
            )
        # self._consumer.stop()
        # self._publisher.stop(delayed=False)
        # self._publisher.stop(delayed=True)
        self._publisher.flush()
        self.assertEqual(1,1)





from pika_client.consumer import AsyncConsumer
from pika_client.publisher import AsyncMemQueuePublisher
from pika_client.launcher import Launcher


def _make_launcher_proxy(connector, setup_config):
    launcher = Launcher(connector, setup_config)
    return launcher


def create_persistent_async_consumer(connection_parameters, setup_config, on_message_callback, **kwargs):
    queue = setup_config.get("queue")
    connector = AsyncConsumer(connection_parameters, on_message_callback, queue=queue, **kwargs)
    return _make_launcher_proxy(connector, setup_config)


def create_persistent_async_publisher(connection_parameters, setup_config, **kwargs):
    connector = AsyncMemQueuePublisher(connection_parameters, **kwargs)
    return _make_launcher_proxy(connector, setup_config)



import threading
import time


class RouteSetup(object):
    def __init__(self, setup_config, channel, on_complete):
        self._setup_config = setup_config
        self._channel = channel
        self._on_complete = on_complete
        self._task_list = self.make_task_list(self._setup_config)
        self._task_iter = iter(self._task_list)

    @staticmethod
    def make_task_list(_setup_config):
        task_list = []
        exc = _setup_config.get("exchange")
        if exc is not None and exc != "":
            task_list.append(("exchange", exc))

        queue = _setup_config.get("queue")
        if queue is not None:
            task_list.append(("queue", queue))

        rk = _setup_config.get("routing_key")
        if rk is not None:
            if exc is None or exc == "":
                if queue is None:
                    task_list.append(("queue", rk))
        return task_list

    def _next_task(self, method_frame):
        try:
            task, name = self._task_iter.__next__()
            self._process_task(task, name)
        except StopIteration:
            self._on_complete()

    def _process_task(self, element, name):
        if element == "exchange":
            self._setup_exchange(name)
        elif element == "queue":
            self._setup_queue(name)
        else:
            raise NotImplementedError("")

    def _setup_exchange(self, exc, exc_type="direct"):
        self._channel.exchange_declare(self._next_task, exc, exc_type)

    def _setup_queue(self, queue, durable=True, passive=False):
        self._channel.queue_declare(self._next_task, queue, durable=durable, passive=passive)

    def setup(self):
        self._next_task(None)



import pika
from pika_client.connector import _AsyncConnector
import queue
import json
import threading


class AsyncMemQueuePublisher(_AsyncConnector):
    def __init__(self, connection_parameters, **kwargs):
        self._mem_q = kwargs.get("mem_q") or queue.Queue(maxsize=kwargs.get("maxsize", 0))
        super(AsyncMemQueuePublisher, self).__init__(connection_parameters, **kwargs)

        self._delay_ms = kwargs.get("delay_ms", 10)
        self._empty_delay_ms = kwargs.get("empty_delay_ms", 1000)
        self._default_exc = kwargs.get("exchange")
        self._default_rk = kwargs.get("routing_key")
        self._deliveries = []
        self._acked = 0
        self._nacked = 0
        self._message_number = 0
        self._delayed_stop = False
        self._flush = False
        self._flush_lock = threading.Lock()

    def start_operation(self):
        self._start_publish(self._channel)

    def send(self, message, exchange=None, routing_key=None):
        if exchange is None:
            if self._default_exc is not None:
                exchange = self._default_exc
            else:
                raise ValueError("must specify exchange if default exchange not set.")
        if routing_key is None:
            if self._default_rk is not None:
                routing_key = self._default_rk
            else:
                raise ValueError("must specify routing key if default routing key not set.")
        self._mem_q.put((message, exchange, routing_key))

    def _on_delivery_confirmation(self, method_frame):
        confirmation_type = method_frame.method.NAME.split('.')[1].lower()
        if confirmation_type == 'ack':
            self._acked += 1
        elif confirmation_type == 'nack':
            self._nacked += 1
        self._deliveries.remove(method_frame.method.delivery_tag)

    def _publish(self, m, exc, rk):
        properties = pika.BasicProperties(app_id='example-publisher',
                                          content_type='application/json',
                                          )
        self._channel.basic_publish(exc, rk, json.dumps(m, ensure_ascii=False), properties)
        self._message_number += 1
        self._deliveries.append(self._message_number)

    def _start_publish(self, channel):
        self._channel.confirm_delivery(self._on_delivery_confirmation)
        self._schedule_next_message()

    def _schedule_next_message(self, is_empty=False):
        if self._flush and is_empty:
            self._flush_lock.release()
        self._connection.add_timeout(
            (self._delay_ms if not is_empty else self._empty_delay_ms) / 1000.0,
            self._on_publish_scheduled)

    def _on_publish_scheduled(self):
        if self._stopping:
            return
        try:
            m, exc, rk = self._mem_q.get(block=False)
            self._publish(m, exc, rk)
            self._schedule_next_message()
        except queue.Empty:
            self._schedule_next_message(is_empty=True)

    def stop(self):
        if self._stopping:
            return
        else:
            super(AsyncMemQueuePublisher, self).stop()
            if self._flush:
                self._flush_lock.release()

    def flush(self, timeout=-1):
        self._flush = True
        self._flush_lock.acquire()
        all_flushed = self._flush_lock.acquire(timeout)
        self._flush = False
        return all_flushed



from setuptools import setup, find_packages

setup(name='pika_client',
      version='0.5.1',
      description='Pika Client',
      url='https://github.com/BenjiBackslash/pika_client',
      author='Hanan Wiener',
      author_email='hanan888@gmail.com',
      license='MIT',
      packages=find_packages(exclude=['contrib', 'docs', 'test*']),
      install_requires=['pika==0.12.0'],
      zip_safe=False)
